#ifndef _CR3BPSolverPool_H_
#define _CR3BPSolverPool_H_

#include "main.h"
#include "capd/threading/ThreadPool.h"
#include "capd/threading/SolverPool.h"

// #########################################################################

struct CR3BPSolverPool 
{
  static int threadsNo;
  static capd::threading::SolverPool<DPCR3BP>* pcr3bp;
  static capd::threading::SolverPool<DCR3BP>* cr3bp;
  static capd::threading::SolverPool<IPCR3BP>* ipcr3bp;
  static capd::threading::SolverPool<ICR3BP>* icr3bp;

  static void initSolvers(int thNo = std::thread::hardware_concurrency()){
    capd::vectalg::Multipointer::generateList(7,2);
    threadsNo =  thNo;
    pcr3bp = new capd::threading::SolverPool<DPCR3BP>(threadsNo,capd::threading::SolverFactory<DPCR3BP>());
    cr3bp = new capd::threading::SolverPool<DCR3BP>(threadsNo,capd::threading::SolverFactory<DCR3BP>());
    ipcr3bp = new capd::threading::SolverPool<IPCR3BP>(threadsNo,capd::threading::SolverFactory<IPCR3BP>());
    icr3bp = new capd::threading::SolverPool<ICR3BP>(threadsNo,capd::threading::SolverFactory<ICR3BP>());
  }

  static void releaseSolvers(){
    delete pcr3bp;
    delete cr3bp;
    delete ipcr3bp;
    delete icr3bp;
  }

  static void setMu(double dmu, capd::interval imu){
    for(int i=0;i<threadsNo;++i){
      pcr3bp->getSolver(i).setMu(dmu);
      cr3bp->getSolver(i).setMu(dmu);
      ipcr3bp->getSolver(i).setMu(imu);
      icr3bp->getSolver(i).setMu(imu);
    }
  }

  static void setTolerance(double tol){
    for(int i=0;i<threadsNo;++i){
      ipcr3bp->getSolver(i).solver.setAbsoluteTolerance(tol);
      ipcr3bp->getSolver(i).solver.setRelativeTolerance(tol);

      icr3bp->getSolver(i).solver.setAbsoluteTolerance(tol);
      icr3bp->getSolver(i).solver.setRelativeTolerance(tol);

      icr3bp->getSolver(i).c2solver.setAbsoluteTolerance(tol);
      icr3bp->getSolver(i).c2solver.setRelativeTolerance(tol);

      icr3bp->getSolver(i).cnsolver.setAbsoluteTolerance(tol);
      icr3bp->getSolver(i).cnsolver.setRelativeTolerance(tol);

      icr3bp->getSolver(i).exc2solver.setAbsoluteTolerance(tol);
      icr3bp->getSolver(i).exc2solver.setRelativeTolerance(tol);

      icr3bp->getSolver(i).excnsolver.setAbsoluteTolerance(tol);
      icr3bp->getSolver(i).excnsolver.setRelativeTolerance(tol);
    }
  }

  static void setOrder(int order, int c2SolverOrder, int cnSolverOrder){
    for(int i=0;i<threadsNo;++i){
      pcr3bp->getSolver(i).solver.setOrder(order);
      cr3bp->getSolver(i).solver.setOrder(order);

      ipcr3bp->getSolver(i).solver.setOrder(order);
      
      icr3bp->getSolver(i).solver.setOrder(order);
      icr3bp->getSolver(i).c2solver.setOrder(c2SolverOrder);
      icr3bp->getSolver(i).exsolver.setOrder(order);
      icr3bp->getSolver(i).exc2solver.setOrder(c2SolverOrder);

      icr3bp->getSolver(i).cnsolver.setOrder(cnSolverOrder);
      icr3bp->getSolver(i).excnsolver.setOrder(cnSolverOrder);
    }
  }
};

#endif
