#ifndef __CONFIG_FILE_READER__
#define __CONFIG_FILE_READER__

#include <string>
#include <sstream>
#include <map>
#include <iostream>
#include <fstream>
#include <cstdlib>

class ConfigFileReader
{
public:
  typedef std::string Key;
  typedef std::string Value;
  typedef std::map<Key,Value> ArgTable;
  typedef ArgTable::iterator iterator;

  ConfigFileReader(const char* filename) : filename(filename) {
    std::ifstream in;
    in.open(filename);
    if(!in.is_open()){
      std::cerr << "Error: cannot open config file \"" << filename << "\".\n";
      exit(0);
    }
    
    // the main loop
    std::string line;
    while( in.good() )
    {
      std::getline(in,line);
      if (line[0] == '@')
      {
        line.erase(0,1);
        std::istringstream inputString(line);
        Key key;
        Value value;
        inputString >> key;
        while( isblank(inputString.peek()) )
          inputString.get();
        std::getline(inputString,value);
        m_args[key] = value;
      }
    }
    in.close();
  }
  
  // this function clears the table of parameters
  void clear(){
    m_args.clear();
  }

  template<class V>
  void getValue(Key key, V& v){
    iterator it = m_args.find(key);

    if(it==m_args.end()){
      std::ostringstream out;
      out << "ConfigFileReader::getValue error: key=" << key << " not found in the config file\n";
      throw std::runtime_error(out.str());
    }
    std::istringstream in(it->second);
    in >> v;
  }

  std::string getConfigFileContent() const {
    std::ifstream in;
    in.open(filename);
    if(!in.is_open()){
      std::cerr << "Error: cannot open config file \"" << filename << "\".\n";
      exit(0);
    }
    std::ostringstream out;
    out << in.rdbuf();
    in.close();
    return out.str(); 
  }
  
private:
  ArgTable m_args;
  std::string filename;
};


#endif

