#ifndef __HALO_MAIN_HEADER_FILE__
#define __HALO_MAIN_HEADER_FILE__

#include <cstring>
#include <cstdlib>
#include "CR3BPSolver.h"
#include "Logger.h"
#include "auxil.h"
#include "LyapunovSchmidtReduction.h"
#include "SymBreakingBifurcationHaloAlgs.h"
#include "SymBreakingBifurcationLyapAlgs.h"

#define CHECK_AND_PRINT(m,x) { bool t=x; cout << m << (#x) << ": " << boolalpha << t << "\n############################\n\n" << flush; if(!t) exit(0); } 

typedef capd::geomset::AffineSet<capd::IMatrix> AffineSet;

// defined in lyapunov.cpp
bool proveLyapunovOrbit(IPCR3BP& pcr3bp, capd::IVector u0, capd::IVector& box, capd::interval mu);
bool proveBranchLyapunovOrbits(IPCR3BP& pcr3bp, double x, double dy, capd::IMatrix C, capd::IVector& box, capd::interval mu);

// defined in halo.cpp
capd::LDVector findHaloOrbit(DCR3BP& cr3bp, capd::LDVector v, long double JacobiConstant);
capd::LDVector computeMuTangentVector(DCR3BP& cr3bp, capd::LDVector h, capd::interval mu);

template<class CR3BP, class V>
V findHaloOrbit(CR3BP& cr3bp, V v, int p, int q, int pi1, int pi2,double tol=1e-13);
template<class CR3BP, class V>
V findBifHaloOrbit(CR3BP& cr3bp, V v, typename CR3BP::Matrix* D=nullptr);

bool proveBifHaloOrbit(ICR3BP& cr3bp, capd::IVector x0, capd::IVector& box, capd::interval mu);
bool proveBranchBifHaloOrbits(DCR3BP& cr3bp, ICR3BP& icr3bp, capd::LDVector x0, capd::IMatrix C, capd::IVector& box, capd::interval mu);
bool checkConvexity(ICR3BP& icr3bp, capd::LDVector x0, capd::IMatrix C, capd::IVector& box, capd::interval mu);

bool proveHaloOrbit(ICR3BP& cr3bp, capd::IVector x0, int p, int q, double sp, double sq, capd::IVector& box, int pi1, int pi2);
bool proveBranchHaloOrbits(DCR3BP& dcr3bp, ICR3BP& cr3bp, capd::LDVector v, int i, int j, int c, capd::interval& si, capd::interval& sj, capd::interval sc, AffineSet& bound, int pi1, int pi2, capd::IVector* Dfp=nullptr);
template<class PM, class C2PM, class C1Set, class C2Set, class Vector>
bool proveBranchHaloOrbits(PM& pm, C2PM& c2pm, C1Set& s0, C2Set& s, Vector& r, int i, int j, int c, int pi1, int pi2, Vector* Dfp=nullptr);

// #########################################################################
// defined in HaloSeedPointsReader.cpp

typedef bool (*Predicate)(long double, long double, long double, long double);

struct HaloSeedPointsReader{
  static void readBifPoints(const char* filename);
  static void readHaloBranch(const char* filename);

  static capd::LDVector getBifHaloOrbit(DCR3BP& cr3bp, long double mu);
  static capd::LDVector getHaloSeedPoint(Predicate predicate, long double p, int i, int j, int c);
  static capd::LDVector getHaloOrbit(DCR3BP& cr3bp, Predicate predicate, long double p, int i, int j, int c, int pi1, int pi2);
  static const std::vector<capd::LDVector>& getHaloBranch() { return haloBranch; }
private:
  static std::vector<capd::LDVector> bifPoints;
  static std::vector<capd::LDVector> haloBranch;
};

#endif
