#include <iostream>
#include <list>
#include <cstring>
#include "SymBreakingBifurcationTask.h"
#include "tictac.h"

using namespace capd;
using namespace std;

void createTasks(std::vector<BaseTask*>& tasks, double L, double maxR, double sx, double sdy, double sz, double stepMu, double deltaMu){
  do{
    double R = capd::min(maxR,L+deltaMu);
    interval mu(L,R);
    tasks.push_back(new SymBreakingBifurcationTask(mu,stepMu,tasks.size()+1,sx,sdy,sz));
    L = R;
  }while(L<maxR);
}

// #########################################################################

void runTasks(std::vector<BaseTask*>& tasks, const char* message){
  auto t = tic(message); 
  BaseTask::runTasks(tasks,BaseTask::FORWARD);
  tac(t);
  BaseTask::releaseTasksMemory(tasks);
}

// #########################################################################

void checkL1(){
  CR3BPSolverPool::setOrder(20,20,5);
  HaloSeedPointsReader::readBifPoints("./dat/symBreakingL1.dat");

  std::vector<BaseTask*> tasks;
  const double sz = (interval(1)/interval(1e+7)).rightBound();
  const double L = (interval(95)/interval(1e+5)).leftBound();

  createTasks(tasks,L,0.01,1e-6,1e-6,sz,1e-6,1e-5); 
  createTasks(tasks,0.01,0.0625,2e-6,1e-6,sz,1e-5,5e-4); 
  createTasks(tasks,0.0625,0.125,4e-6,4e-6,sz,2e-5,1e-3); 
  createTasks(tasks,0.125,0.25,5e-6,5e-6,sz,3e-5,1e-3);
  createTasks(tasks,0.25,0.5,6e-6,6e-6,sz,4e-5,1e-3);

  runTasks(tasks,"L1 symmetry breaking bifurcations for mu\\in[9.5e-4,0.5] and |z|<=1e-7");
}

// #########################################################################

void checkL2(){
  CR3BPSolverPool::setOrder(20,8,8);
  HaloSeedPointsReader::readBifPoints("./dat/symBreakingL2.dat");

  std::vector<BaseTask*> tasks;
  const double sz = (interval(1)/interval(1e+7)).rightBound();
  const double L = (interval(95)/interval(1e+5)).leftBound();

  createTasks(tasks,L,0.01,1e-6,1e-6,sz,1e-6,1e-5); 
  createTasks(tasks,0.01,0.0625,2e-6,1e-6,sz,1e-5,5e-4); 
  createTasks(tasks,0.0625,0.125,5e-6,5e-6,sz,2e-5,1e-3); 
  createTasks(tasks,0.125,0.25,5e-6,5e-6,sz,5e-5,1e-3);
  createTasks(tasks,0.25,0.5,1e-5,1e-5,sz,1e-4,1e-3);

  runTasks(tasks,"L2 symmetry breaking bifurcations for mu\\in[9.5e-4,0.5] and |z|<=1e-7");
}

// #########################################################################

void checkL3(){
  CR3BPSolverPool::setOrder(20,12,8);
  HaloSeedPointsReader::readBifPoints("./dat/symBreakingL3.dat");

  std::vector<BaseTask*> tasks;
  const double sz = (interval(1)/interval(1e+7)).rightBound();
  const double L = (interval(95)/interval(1e+5)).leftBound();

  createTasks(tasks,L,0.002,1e-8,1e-8,sz,1e-8,1e-6); 
  createTasks(tasks,0.002,0.003,1e-8,1e-8,sz,2e-8,1e-6); 
  createTasks(tasks,0.003,0.005,1e-8,1e-8,sz,3e-8,1e-6); 
  createTasks(tasks,0.005,0.007,1e-8,1e-8,sz,5e-8,1e-6); 
  createTasks(tasks,0.007,0.01,1e-8,1e-8,sz,8e-8,1e-6); 
  createTasks(tasks,0.01,0.0625,1e-8,1e-8,sz,4e-7,2e-4); 
  createTasks(tasks,0.0625,0.125,1e-8,1e-8,sz,1e-6,1e-3); 
  createTasks(tasks,0.125,0.25,2e-8,2e-8,sz,2e-6,1e-3);
  createTasks(tasks,0.25,0.5,2e-7,2e-7,sz,8e-6,1e-3); 

  runTasks(tasks,"L3 symmetry breaking bifurcations for mu\\in[9.5e-4,0.5] and |z|<=1e-7");
}

// #########################################################################

int main(int argc, char* argv[]){
  cout.precision(17);
  CR3BPSolverPool::initSolvers();

  int i = 0;
  if(argc>1)
    i = atoi(argv[1]);

  switch(i){
    case 1: checkL1(); break;
    case 2: checkL2(); break;
    case 3: checkL3(); break;
    default: 
      checkL1();
      checkL2();
      checkL3();
  }

  CR3BPSolverPool::releaseSolvers();

  return 0;
}
