#include <iostream>
#include <cstring>
#include "BaseTask.h"
using namespace capd;
using namespace std;

std::mutex BaseTask::consoleGuard;
std::vector<BaseTask*>* BaseTask::tasks = nullptr;

BaseTask::BaseTask(interval parameterRange, double step, int taskNo)
  : parameterRange(parameterRange), stepParameter(step), taskNo(taskNo)
{}

// #########################################################################
  
void BaseTask::report(){
  bool success = (this->parameterRange.leftBound()==this->validatedRange.leftBound() and this->parameterRange.rightBound()==this->validatedRange.rightBound());
  {
    std::lock_guard<std::mutex> lock(consoleGuard);
    cout << "Task #" << taskNo << " completed. Result=" << boolalpha << success << ". Condition checked on interval: ";
    if(this->result)
      cout << this->validatedRange;
    else
      cout << "emptySet";
    cout << endl;
  }
  completed = true;
}

// #########################################################################

void BaseTask::runTasks(std::vector<BaseTask*>& tasks, ExecutionOrder order){
  BaseTask::tasks = &tasks;
  capd::threading::ThreadPool threadPool(CR3BPSolverPool::threadsNo);
  cout << "Total number of tasks: " << tasks.size() << endl;

  if(order == REVERSE)
    for(auto t = tasks.rbegin();t!=tasks.rend();++t) threadPool.process(*t);
  else
    for(auto t = tasks.begin();t!=tasks.end();++t) threadPool.process(*t);
    
  for(BaseTask* t : tasks) t->join();
  BaseTask::tasks = nullptr;
  
  // merge results from tasks
  std::list<interval> list;
  interval prev = 0.;
  interval tmp;
  bool first = true;
  for(BaseTask* t : tasks) {
    if(t->result) {
      if(first){
        first = false;
        prev = t->validatedRange;
      } else {
        if(intersection(prev,t->validatedRange,tmp))
          prev = intervalHull(t->validatedRange,prev);
        else {
          list.push_back(prev);
          prev = t->validatedRange;
        }
      }
    }
  }
  if(prev!=0) list.push_back(prev);
  
  cout.precision(17);
  cout << "All tasks completed. Required condition validated on the parameter range:\n";
  if(!list.empty()){
    for(auto param : list) cout << param << endl;
  }
  else
    cout << "emptySet\n";
}

// #########################################################################

void BaseTask::releaseTasksMemory(std::vector<BaseTask*>& tasks){
  for(BaseTask* t : tasks) delete t;
}
