#include <iostream>
#include <fstream>
#include "main.h"
using namespace capd;

// #########################################################################

std::vector<capd::LDVector> HaloSeedPointsReader::bifPoints;
std::vector<capd::LDVector> HaloSeedPointsReader::haloBranch;

// #########################################################################

void HaloSeedPointsReader::readBifPoints(const char* filename){
  bifPoints.clear();
  std::ifstream in(filename);
  double m,x,dy,jc;
  while(true){
    in >> m >> x >> dy >> jc;
    if(in.good())
      bifPoints.push_back(LDVector({m,x,dy,jc}));
    else break;
  }
  in.close();
}

// #########################################################################

void HaloSeedPointsReader::readHaloBranch(const char* filename){
  haloBranch.clear();
  std::ifstream in(filename);
  double m,x,dy,jc;
  LDVector u(6);
  int tmp;
  while(true){
    in >> u[0] >> u[2] >> u[4];
    if(in.good())
      haloBranch.push_back(u);
    else break;
  }
  in.close();
}

// #########################################################################

LDVector HaloSeedPointsReader::getBifHaloOrbit(DCR3BP& cr3bp, long double mu){
  // bisection search for nearby two values of mu in the database
  int i=0, j=bifPoints.size()-1;
  while(i+1<j){
    int k=(i+j)/2;
    if(mu<bifPoints[k][0]) j=k;
    else if(mu>bifPoints[k][0]) i=k;
    else return LDVector({bifPoints[k][1],0.,0.,0.,bifPoints[k][2],0.});
  }

  // secant approximation of the result
  long double t = (mu-bifPoints[i][0])/(bifPoints[j][0]-bifPoints[i][0]);
  long double x = t*bifPoints[j][1] + (1-t)*bifPoints[i][1];
  long double dy = t*bifPoints[j][2] + (1-t)*bifPoints[i][2];
  
  LDVector halo({x,0.,0.,0.,dy,0.});
  cr3bp.setMu(mu);
  // refine by Newton method
  return findBifHaloOrbit(cr3bp,halo);
}

// #########################################################################

LDVector HaloSeedPointsReader::getHaloSeedPoint(Predicate predicate, long double p, int i, int j, int c){
  // bisection search for the point when predicate is not satisfied
  int m=0, n=haloBranch.size()-1;
  while(m+1<n){
    int k=(n+m)/2;
    if(predicate(p,haloBranch[k][0],haloBranch[k][2],haloBranch[k][4])) m=k;
    else n=k;
  }

  // secant approximation of the result
  long double t = (p-haloBranch[m][c])/(haloBranch[n][c]-haloBranch[m][c]);
  LDVector u(6);
  u[i] = t*haloBranch[n][i] + (1-t)*haloBranch[m][i];
  u[j] = t*haloBranch[n][j] + (1-t)*haloBranch[m][j];
  u[c] = p;

  PRINT(u);
  return u;  
}

// #########################################################################

LDVector HaloSeedPointsReader::getHaloOrbit(DCR3BP& cr3bp, Predicate predicate, long double p, int i, int j, int c, int pi1, int pi2){
  return findHaloOrbit(cr3bp,getHaloSeedPoint(predicate,p,i,j,c),i,j,pi1,pi2);
}
