#include <iostream>
#include <list>
#include <cstring>
#include "SymBreakingBifurcationTask.h"
using namespace capd;
using namespace std;

// #########################################################################

SymBreakingBifurcationTask::SymBreakingBifurcationTask(interval mu, double smu, int taskNo, double sx, double sdy, double sz)
  : SymBreakingBifurcationLyapAlgs(5),
    SymBreakingBifurcationHaloAlgs(7),
    BaseTask(mu,smu,taskNo),
    sizeX(sx), sizeDY(sdy), sizeZ(sz)
{}
 
// #########################################################################

  
void SymBreakingBifurcationTask::run(unsigned id){    
  double firstMu = this->parameterRange.rightBound();
  double lastMu = this->parameterRange.leftBound(); 
  double currentMu = firstMu;
  this->validatedRange = firstMu;
  
  IPCR3BP& ipcr3bp = CR3BPSolverPool::ipcr3bp->getSolver(id);
  ICR3BP& icr3bp = CR3BPSolverPool::icr3bp->getSolver(id);
  DCR3BP& cr3bp = CR3BPSolverPool::cr3bp->getSolver(id);

  LDMatrix D(6,6);       // approximate derivative at bifurcatiion point. Used to choose coordinate system for rigorous proof.

  interval I = interval(-1,1), x, dy;
  IVector lbox({I*sizeX,0.,0.,I*sizeDY});
  IVector hbox({I*sizeX,0.,interval(0,1)*sizeZ,0.,I*sizeDY,0.});

  // first seed point for bifurcation is read from a file with precomputed data
  LDVector haloBif = HaloSeedPointsReader::getBifHaloOrbit(cr3bp,currentMu);
  do{  
    double nextMu = capd::max(currentMu - this->stepParameter,lastMu);
    interval m = interval(nextMu,currentMu);
    bool oneStepResult = false;
    try{
      // find candidate and prepare coordinate system
      cr3bp.setMu((nextMu+currentMu)/2);
      haloBif = findBifHaloOrbit(cr3bp,haloBif,&D);
      LDVector t = computeMuTangentVector(cr3bp,haloBif,m);
      
      this->SymBreakingBifurcationHaloAlgs::init(haloBif,D,t,hbox,m);
      this->SymBreakingBifurcationLyapAlgs::init(haloBif,D,t,lbox,m);

      oneStepResult = this->proveBranchLyapunovOrbits(ipcr3bp) and
                      this->proveBranchBifHaloOrbits(icr3bp);

      // if both branches are validated, then intersection of Newton operator from both routines is a bound on bifurcation point.
      if(oneStepResult){
        oneStepResult = false;
        interval x = SymBreakingBifurcationHaloAlgs::N[0];
        if(!intersection(SymBreakingBifurcationHaloAlgs::N[1],SymBreakingBifurcationLyapAlgs::N,dy)){
          cout << "Fatal error in the intersection of two Newton operators. This should not happen\n. Stoppint the programm.";
          exit(0);
        }
        // check remaining conditions for SymBreaking bifurcation
        oneStepResult = checkMonotonicity(ipcr3bp,x,dy) and checkConvexity(icr3bp,x,dy);        
      }
    }catch(...) {}

    if(oneStepResult){
      currentMu = nextMu;
      this->validatedRange = interval(currentMu,firstMu);
      this->result = true;  
    } else {
      this->stepParameter *= 0.9;
      FLUSH("Decsreasing stepMu. New stepMu="<< this->stepParameter);
    }
  } while(currentMu>lastMu and this->stepParameter>1e-12);

  this->report();
} // run

